﻿using System;
using System.Threading;
using System.Threading.Tasks;
using WatcherCommons.Interfaces;
using KinectProtocol.Properties;

namespace KinectProtocol
{
    public class KinectTrigger : ITrigger
    {
        private int _sens = 25;
        private Task _thTrigger = null;
        private bool _bRunning = false;
        private bool _bSource = false;
        private int _threshold = 100;
        private ushort[] _oldFrame = null;
        public KinectTrigger()
        {
        }
        public event EventHandler OnTriggerFired = null;
        public string ConnectionString
        {
            get
            {
                return string.Join(";", new string[] { "sens=" + _sens.ToString(), "source=" + (_bSource ? "depth" : "body"),  "thres="  +_threshold.ToString()});
            }
            set
            {
                foreach (string part in value.Split(';'))
                {
                    string[] p = part.Split('=');
                    switch (p[0])
                    {
                        case "sens":
                            int.TryParse(p[1], out _sens);
                            if (_sens > 100)
                            {
                                _sens = 100;
                            }
                            if (_sens < 0)
                            {
                                _sens = 0;
                            }
                            break;
                        case "source":
                            _bSource = p[1] == "depth";
                            break;
                        case "thres":
                            int.TryParse(p[1], out _threshold);
                            if (_threshold <= 10)
                            {
                                _threshold = 10;
                            }
                            break;
                    }
                }
            }
        }
        public void Initialize()
        {
            OnTriggerFired = null;
            Stop();
        }
        public void Start()
        {
            if (_thTrigger == null)
            {
                Sensor.OpenSensor(_bSource ? SensorCaps.Depth : SensorCaps.Body);
                _bRunning = true;
                _oldFrame = null;
                _thTrigger = Task.Run(() => { TriggerTest(); });
            }
        }
        public void Stop()
        {
            try
            {
                if (_thTrigger != null)
                {                    
                    _bRunning = false;
                    Task.WaitAll(new Task[] { _thTrigger });
                    Sensor.CloseSensor();
                    _thTrigger.Dispose();
                    _thTrigger = null;
                }
            }
            catch
            {
            }
        }
        private void TriggerTest()
        {
            while (_bRunning && (OnTriggerFired != null))
            {
                try
                {
                    if (_bSource)
                    { 
                        ushort[] frame = Sensor.DepthFrame;
                        if (frame != null)
                        {
                            if (_oldFrame != null)
                            {
                                int max = (_sens * frame.Length) / 100;
                                if (max > 0)
                                {
                                    int cnt = 0;
                                    for (int ix = 0; ix < frame.Length; ix++)
                                    {
                                        if (Math.Abs(frame[ix] - _oldFrame[ix]) > _threshold)
                                        {
                                            cnt++;
                                            if (cnt >= max)
                                            {
                                                OnTriggerFired?.Invoke(this, EventArgs.Empty);
                                                Thread.Sleep(2000);
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                            _oldFrame = frame;
                        }
                    }
                    else
                    {
                        if (Sensor.BodyTracked)
                        {
                            OnTriggerFired?.Invoke(this, EventArgs.Empty);
                            Thread.Sleep(2000);
                        }
                    }
                }
                catch
                {
                }
            }
        }
        public override string ToString()
        {
            return Resources.ID_TRIGPROTOCOL;
        }
    }
}
